<?php 

defined('ABSPATH') || exit;

/**
 * Main FSSC Metabox Class. 
 *
 * @class FSSC_Metabox
 */
class FSSC_Metabox extends FSSC_Action{
	function __construct() {
		add_action( 'admin_footer', array( $this, 'admin_enqueue_scripts' ) );
		add_action( 'add_meta_boxes', array($this,'fssc_display_metabox'));
		add_action( 'save_post', array( $this, 'fssc_save_schema_data' ), 10, 3 );
	}

	function admin_enqueue_scripts() {
		wp_register_style( 'fssc-style-css', FSSC_BASE_URL.'assets/css/fssc-style.css' );
		wp_enqueue_style( 'fssc-style-css' );
		wp_register_script( 'fssc-script', FSSC_BASE_URL.'assets/js/fssc-common.js' );
		wp_enqueue_script( 'fssc-script' );
	}

	function fssc_display_metabox(){
		$fsSchemaFunction = new FSSC_SchemaFunction;
     	$screens = $fsSchemaFunction->getSchemaSettingsPost();
		add_meta_box( 'schema-metabox-fields', 'Schema Details', array($this,'fssc_display_schema_fields'), $screens, 'advanced', 'high', null );
	}

	// AutoFill product data
    function getProductAutofill($tabID, $fieldId, $data){
    	global $post;
    	$data['fieldId'] = $fieldId;
		$data['id']      = $tabID . "_" . $fieldId;
		$data['name']    = $tabID . "[{$fieldId}]";
		$product      = $post->ID;
		$product_details = new WC_Product($product);
		if($data['name'] == 'fssc_product[active]'){
			$data['value']   = "1";
		}
		if($data['name'] == 'fssc_product[name]'){
			$data['value']   = $product_details->get_name();
		}
		if($data['name'] == 'fssc_product[image]'){
			$data['value']   = $product_details->get_image_id();
		}
		if($data['name'] == 'fssc_product[description]'){
			$data['value'] = $product_details->get_description();
		}
		if($data['name'] == 'fssc_product[sku]'){
			$data['value'] = $product_details->get_sku();
			if(!$product_details->get_sku()){
			    $data['value'] = $product;
			}
		}
		if($data['name'] == 'fssc_product[reviewRatingValue]'){
			$data['value'] = $product_details->get_average_rating();
		}
		if($data['name'] == 'fssc_product[ratingCount]'){
			$data['value'] = $product_details->get_review_count();
		}
		if($data['name'] == 'fssc_product[ratingValue]'){
			$data['value'] = $product_details->get_average_rating();
		}
		if($data['name'] == 'fssc_product[reviewAuthor]'){
			$auth = get_post($product);
			$authid = $auth->post_author;
			$data['value'] = get_the_author_meta('user_nicename',$authid);
		}
		if($data['name'] == 'fssc_product[reviewAuthor]'){
			$auth = get_post($product);
			$authid = $auth->post_author;
			$data['value'] = get_the_author_meta('user_nicename',$authid);
		}
		if($data['name'] == 'fssc_product[price]'){
			$data['value'] = $product_details->get_price();
		}
		if($data['name'] == 'fssc_product[priceValidUntil]'){
			$data['value'] = $product_details->get_date_on_sale_to();
		}
		if($data['name'] == 'fssc_product[url]'){
			$data['value'] = get_permalink( $product);
		}
		if($data['name'] == 'fssc_product[priceCurrency]'){
			$data['value'] = get_option('woocommerce_currency');
		}
		if($data['name'] == 'fssc_product[availability]'){
			$availibility=FSSC_SchemaFunction::getProductAvailibilityList($product_details->get_stock_status());
			$data['value'] = $availibility;
		}
		return $data;
    }  

    // Display fields with data
	function fssc_display_schema_fields( $post ){
  		$schemaAction 	= FSSC_Metabox::fssc_ck();
    	$current_id 	= $post->ID;	
		$displayclass   = $html = "";
		$schemaField 	= new FSSC_Fields;
		$schemaFields 	= $schemaField->getSchemaFields();
		$customSchema   = get_post_meta( $post->ID, "_fssc_custom_schema", true );
		$html .= '<div class="meta-tab-wrapper">
			<div class="fs-custom-schema">
				<label class="field-label">Custom schema ( Put custom code of schema here )</label>
				<textarea class="fs-custom-code" name="fs_custom_schema">'. wp_kses($customSchema, array()) .'</textarea>
			</div>
			<div class="meta-tab-inner">
		  	<ul class="tab">';
		  	foreach($schemaFields as $field => $value){
		  		$tabID 		= 'fssc_'. $field;
		  		$metaData 	= get_post_meta( $post->ID, $tabID, true );
		  		$activeClass = ($field == 'article') ? 'isActive' : '';
		  		$html .= '<li data-id="'.$tabID.'" class="tablink '.$activeClass.'"><span class="tablink-text">'.$value['title'].'</span></li>';
		  	}
		$html .= '</ul>';
		$htmlCont = "";
		$fieldvalue = "";
		foreach($schemaFields as $key => $schema){
			$tabID 	= 'fssc_'. $key;
			$metaData = get_post_meta( $post->ID, $tabID, true );
			$metaData = ( is_array( $metaData ) ? $metaData : array() );
			$html .= '<div id="'.$tabID.'" class="tabcontent">
  				<h3>'.$schema["title"].'</h3>';
  			$html .= '<div class="schema-fields-wrapper">';
			if ( ! empty( $schema['fields'] ) ) {
					foreach ( $schema['fields'] as $fieldId => $data ) {
						if(!empty($metaData)){
							$fieldvalue =  $metaData[ $fieldId ];
						}
						if($schema["title"] == 'FAQ' && $fieldId != 'isActive' && is_array($fieldvalue) ){
						$html .= '<div class="faq-wrapper">'; 	
					}
						if(is_array($fieldvalue) && $schema["title"] == 'FAQ'){
							foreach($fieldvalue as $key => $v){
								foreach($v as $k => $j){
									$data['fieldId'] = $fieldId;
							    $data['id']      = $tabID . "_" . $fieldId;
  								if($k =='question'){
  									$data['type']    = "text";
  									$data['title']   = 'question';
  									$data['name']    = $tabID . "[{$fieldId}][$key][question]";
  								}
  								if($k =='answer'){
  									$data['type']    = "textarea";
  									$data['title']   = 'answer';
  									$data['name']    = $tabID . "[{$fieldId}][$key][answer]";
  								}
  								$data['value']   = $j;
  							
  								$html .= $this->getSchemaField($data, $fieldId);
								}
								
							}
						}
						// Product Autofill or not
						else if($schema["title"]=="Product" && !is_array($fieldvalue) && class_exists( 'woocommerce')){
								$data = $this->getProductAutofill($tabID, $fieldId, $data);
								if(!$data['value']){
								  $data['fieldId'] = $fieldId;
							  $data['id']      = $tabID . "_" . $fieldId;
							  $data['name']    = $tabID . "[{$fieldId}]"; 
							  if($fieldvalue){
							     $data['value']   = $fieldvalue;
						      }
								}
							
							$html .= $this->getSchemaField($data, $fieldId);
						}
						else{
							$data['fieldId'] = $fieldId;
							$data['id']      = $tabID . "_" . $fieldId;
							$data['name']    = $tabID . "[{$fieldId}]";
							if($fieldvalue){
								$data['value']   = $fieldvalue;
							}
							$html .= $this->getSchemaField($data, $fieldId);
						}
					if( $schema["title"] == 'FAQ' && $fieldId != 'active' && is_array($fieldvalue) ){
						$html .= '</div>'; 	
					}
				}
			}
  			$html .= '</div>';
  			if($schema["title"] == 'FAQ'){
				$html .= '<div class="add-more-wrap"><button type="button" class="add-more-button" data-limit="5">'.__("Add FAQ", "fs-schema-wordpress").'</button></div>';	
			}
			$html .= '</div>';
		}
		$html .= '</div></div>';
		echo $html;
	}

	// Display HTML of fields
	function getSchemaField($data, $fieldId){
		$html 	= '';
		$holderClass = '';
		$id 	= $data['id'];
        $name 	= $data['name'];
        $value  = isset($data['value']) ? $data['value'] : '';
        $class 	= isset($data['class']) ? ($data['class'] ? $data['class'] : null) : null;
		$title 	= (isset($data['title']) ? ($data['title'] ? $data['title'] : null) : null);
        $desc 	= (isset($data['desc']) ? ($data['desc'] ? $data['desc'] : null) : null);
        $require = (isset($data['required']) ? ($data['required'] ? sprintf('<span class="required">*</span>') : null) : null);
		switch ($data['type']) {
			case 'text':
                $html .= "<input type='text' id='{$id}' class='{$class}' name='{$name}' value='" . esc_html($value) . "' />";
                break;
            case 'textarea':
                $html .= "<textarea id='{$id}' class='{$class}' name='{$name}' >" . wp_kses($value, array()) . "</textarea>";
                break;
            case 'number':
                if ($data['fieldId'] == 'price') {
                    $html .= "<input type='number' step='any' id='{$id}' class='{$class}' name='{$name}' value='" . esc_attr($value) . "' />";
                } else {
                    $html .= "<input type='number' id='{$id}' class='{$class}' name='{$name}' value='" . esc_attr($value) . "' />";
                }
                break;
            case 'image':
            		$html .= '<div class="fssc-image">';
                    $ImageId = !empty($value) ? absint($value) : 0;
                    $image = $imgInfo = null;
                    if ($ImageId) {
                        $image 		= wp_get_attachment_image($ImageId, "medium");
                        $imgData    = wp_get_attachment_metadata( $ImageId );
                        $imgInfo   .= "<div class='image-info'><span>".(! empty( $imgData['width'] ) ? absint( $imgData['width'] ) : 0)."px</span> - ";
                        $imgInfo   .= " <span>".(! empty( $imgData['height'] ) ? absint( $imgData['height'] ) : 0)."px</span></div>";
                    }
            		if(isset($data['use'])){
            			$html .= "<div class='fssc-image-wrapper'>";
	                    $html .= '<span class="fsscAddLogo"><span class="dashicons dashicons-plus-alt"></span></span>';
	                    $html .= '<span class="fsscRemoveLogo ' . ($image ? null : "fssc-hidden") . '"><span class="dashicons dashicons-trash"></span></span>';
	                    $html .= '<div class="fssc-image-preview">' . $image . '</div>';
	                    $html .= "<input type='hidden' name='{$name}' value='" . absint($ImageId) . "' />";
	                    $html .= "</div>";
            		}else{
            			$html .= "<div class='fssc-image-wrapper'>
                    			<span class='fsscAddImg'>
                    				<span class='dashicons dashicons-plus-alt'></span>
                    			</span>";
	                    $html .= '<span class="fsscRemoveImg ' . ($image ? null : "fssc-hidden") . '"><span class="dashicons dashicons-trash"></span></span>';
	                    $html .= '<div class="fssc-image-preview" style="background-image: url('.wp_get_attachment_url( $ImageId, 'full' ).')"></div>';
	                    $html .= "<input type='hidden' name='{$name}' value='" . absint($ImageId) . "'/>";
	                    $html .= "{$imgInfo}";
	                    $html .= "</div>";
            		}
                    $html .= "</div>";
                    break;
            case 'select':
                $html .= "<select name='{$name}' class='select2 {$class}' id='{$id}'>";
                if (!empty($data['empty'])) {
                    $html .= "<option value=''>{$data['empty']}</option>";
                }
                if (!empty($data['options']) && is_array($data['options'])) {
                    if ($this->isAssoc($data['options'])) {
                        foreach ($data['options'] as $optKey => $optValue) {
                            $slt = ($optKey == $value ? "selected" : null);
                            $html .= "<option value='" . esc_attr($optKey) . "' {$slt}>" . esc_html($optValue) . "</option>";
                        }
                    } else {
                        foreach ($data['options'] as $optValue) {
                            $slt = ($optValue == $value ? "selected" : null);
                            $html .= "<option value='" . esc_attr($optValue) . "' {$slt}>" . esc_html($optValue) . "</option>";
                        }
                    }
                }
                $html .= "</select>";
                break;
            case 'checkbox':
                $checked = ($value ? "checked" : null);
                $html .= "<div class='fssc-checkbox-wrapper'>";
                $html .= "<label class='fssc-switch--toggle' for='{$id}'><input type='checkbox' id='{$id}' class='{$class}' name='{$name}' {$checked} value='1' /><span class='switch-box'></span></label>";
                $html .= "</div>";
                break;
            default:
                $html .= "<input id='{$id}' type='{$data['type']}' value='" . esc_attr($value) . "' name='$name' />";
                break;
		}
		$field_html = sprintf('<div class="field-content">%s<p class="description">%s</div>', $html, $desc);
		$label 		= "<label class='field-label' for=''>{$title}{$require}</label>";
		if ($data['type'] == 'heading') {
            $holderClass .= 'heading-container';
            $label = '';
            $field_html = sprintf('<div class="section-title-wrap">%s%s</div>',
                $title ? sprintf('<h5>%s</h5>', $title) : '',
                $desc ? sprintf('<p class="description">%s</p>', $desc) : null
            );
        }
        if($data['type'] == 'list_item'){
        	$label = '';
			$field_html = '';
        }
		$html 	= sprintf('<div class="field-container %s">%s%s</div>',
						$holderClass,
                		$label,
                		$field_html
            		);
		return $html;
	}

	// Save Meta value
	function fssc_save_schema_data($post_id, $post, $update){
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
				return $post_id;
		}

		if(isset($_GET['post_type'])) {
			$post_type = $_GET['post_type'];
		}
		// Check permissions
		if ( ! empty( $post_type ) ) {
			if ( ! current_user_can( 'edit_' . $post_type, $post_id ) ) {
				return $post_id;
			}
		}

		$meta         = array();
		$schemaModel  = new FSSC_Fields;
		$schemaFields = $schemaModel->getSchemaFields();

		if( ! empty( $schemaFields )){
			foreach ( $schemaFields as $schemaID => $schema ) {
				$schemaMetaId = "fssc_" . $schemaID;
				$data         = array();
				if ( ! empty( $schema['fields'] ) ) {
					foreach ( $schema['fields'] as $fieldId => $fieldData ) {
						$value            = ( ! empty( $_REQUEST[ $schemaMetaId ][ $fieldId ] ) ? fssc_sanitizeInputValue($_REQUEST[ $schemaMetaId ][ $fieldId ],  $_REQUEST[ $schemaMetaId ][ $fieldId ]['type']) : null );
						if(!is_array($value)){
							$value        = $schemaModel->sanitize( $fieldData, $value );
						}
						$data[ $fieldId ] = $value;
					}
				}
				$meta[ $schemaMetaId ] = $data;
			}
		}
		if ( count( $meta ) > 0 ) {
			foreach ( $meta as $mKey => $mValue ) {
				update_post_meta( $post_id, $mKey, $mValue );
			}
		}
	}

	function isAssoc($array) {
        $keys = array_keys($array);
        return $keys !== array_keys($keys);
    }
}
new FSSC_Metabox();